package controllers;

import models.Database;
import models.owl.*;
import models.owl.Class;
import play.mvc.Controller;
import sun.swing.plaf.windows.ClassicSortArrowIcon;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * Контроллер секции учебных групп.
 * @author M. Navrotskiy
 * @version 1.0
 */
public class ClassesSection extends Controller {

    /** Список учебных аудиторий. */
    private static ArrayList<models.owl.Class> classes = new ArrayList<Class>();
    /** Глобальный идентификатор аудиторий. */
    private static int globalID = 0;

    /**
     * Функция отображения главной страницы секции.
     */
    public static void index() {

        classes.clear();
        classes = Database.readClasses();

        renderArgs.put("classes", classes.toArray());
        render(classes.toArray());
    }

    /**
     * Функция отображения страницы создания группы.
     */
    public static void create() {
        render();
    }

    /**
     * Функция отображения страницы удаления группы.
     */
    public static void delete() {

        classes.clear();
        classes = Database.readClasses();

        renderArgs.put("classes", classes.toArray());
        render(classes.toArray());
    }

    /**
     * Функция отображения детальной страницы учебной группы.
     * @param id Идентификатор учебной группы.
     */
    public static void detail(String id) {

        classes.clear();
        classes = Database.readClasses();

        int elementID = Integer.parseInt(id);

        Class c = getClassById(elementID);

        if (c == null)
            Application.page404();
        else {
            renderArgs.put("c", c);
            render(c);
        }
    }

    /**
     * Функция отображения страницы изменения группы.
     * @param id Идентификатор изменяемого элемента.
     */
    public static void edit(String id) {

        classes.clear();
        classes = Database.readClasses();

        int elementID = Integer.parseInt(id);

        Class c = getClassById(elementID);
        if (c == null)
            Application.page404();
        else {
            renderArgs.put("c", c);
            render(c);
        }
    }

    /**
     * Функция изменения учебной группы.
     * @param id Идентификатор учебной группы.
     * @param c_name Новое имя учебной группы.
     */
    public static void editDo (String id, String c_name) {

        classes.clear();
        classes = Database.readClasses();

        int elementID = Integer.parseInt(id);
        Iterator<Class> it = classes.iterator();

        while (it.hasNext()) {
            Class c = it.next();

            if (c.getId() == elementID) {
                c.setName(c_name);
                break;
            }
        }

        index();
    }

    /**
     * Метод изменения группы.
     * @param id Идентификатор группы.
     * @param c_name Название группы.
     */
    public static void save (String id, String c_name) {

        Database.editClass(Integer.parseInt(id), c_name);

        index();
    }

    /**
     * Функция создания группы.
     * @param c_name Название группы.
     */
    public static void create_new (String c_name) {

        Database.addClass(c_name);

        create();
    }

    /**
     * Метод удаления учебной группы по id.
     * @param id Идентификатор учебной группы.
     */
    public static void remove (String id) {

        Database.removeClass(id);

        delete();
    }

    /**
     * Метод удаления всех учебных групп.
     */
    public static void removeAll() {

        classes.clear();
        classes = Database.readClasses();

       classes.clear();

        delete();
    }

    /**
     * Метод поиска группы по идентификатору.
     * @param id Идентификатор искомой учебной группы.
     * @return Найденная учебная группа или null.
     */
    private static Class getClassById (int id) {
        Iterator<Class> it = classes.iterator();
        Class c = null;

        while (it.hasNext()) {
            Class t = it.next();

            if (t.getId() == id) {
                c = t;
                break;
            }
        }

        return c;
    }

}
