package controllers;

import models.Database;
import models.owl.Room;
import play.mvc.Controller;

import java.util.ArrayList;

/**
 * Контроллер секции учебных ауиторий
 * @author M. Navrotskiy
 * @version 1.0
 */
public class RoomsSection extends Controller {

    /**
     * Функция отображения главной страницы секции.
     */
    public static void index() {

        ArrayList<Room> rooms = Database.readRooms();

        renderArgs.put("rooms", rooms.toArray());
        render(rooms.toArray());
    }

    /**
     * Функция отображения страницы создания аудитории.
     */
    public static void create() {
        render();
    }

    /**
     * Функция отображения страницы удаления аудиторий.
     */
    public static void delete() {

        ArrayList<Room> rooms = Database.readRooms();

        renderArgs.put("rooms", rooms.toArray());
        render(rooms.toArray());
    }

    /**
     * Функция удаления аудитории из онтологии.
     * @param id Идентификатор аудитории.
     */
    public static void remove (String id) {

        Database.removeRoom(id);

        delete();
    }

    /**
     * Функция отображения детальной страницы аудитории.
     */
    public static void detail(String id) {

        Room r = Database.getRoomById(Integer.parseInt(id));

        renderArgs.put("r", r);
        render(r);
    }

    /**
     * Функция отображения страницы изменения аудитории.
     */
    public static void edit (String id) {

        Room r = Database.getRoomById(Integer.parseInt(id));

        renderArgs.put("r", r);
        render(r);
    }

    public static void save (String id, String re_number, boolean re_lec) {

        Database.editRoom(Integer.parseInt(id), re_number, re_lec);

        index();
    }

    /**
     * Функция создания учебной аудитории.
     * @param r_name Номер учебной аудитории.
     * @param r_is Признак того, лекционная аудитория или нет.
     */
    public static void createNew (String r_name, boolean r_is) {

        Database.addRoom(r_name, r_is);

        create();
    }

}
