package controllers;

import models.*;
import models.Error;
import models.owl.*;
import models.owl.Class;
import play.mvc.Controller;

import java.util.ArrayList;

/**
 * Класс контроллер секции учебных предметов.
 * @author M. Navrotskiy
 * @version 1.0
 */
public class SubjectsSection extends Controller {

    /**
     * Функция отображения главной страницы секции.
     */
    public static void index() {

        ArrayList<Subject> subjects = Database.readSubjects();

        renderArgs.put("subjects", subjects.toArray());
        render(subjects.toArray());
    }

    public static String SubjectTimeToString (int value) {
        return SubjectTime.toString(value);
    }

    /**
     * Функция отображения формы создания учебного предмета.
     */
    public static void create (String message) {

        ArrayList<Room> rooms = Database.readRooms();
        renderArgs.put("rooms", rooms.toArray());

        ArrayList<Teacher> teachers = Database.readTeachers();
        renderArgs.put("teachers", teachers.toArray());

        ArrayList<String> times = Database.readTime();
        renderArgs.put("times", times.toArray());

        ArrayList<String> weeks = Database.readWeeks();
        renderArgs.put("weeks", weeks.toArray());

        ArrayList<String> days = Database.readDays();
        renderArgs.put("days", days.toArray());

        ArrayList<models.owl.Class> classes = Database.readClasses();
        renderArgs.put("classes", classes.toArray());

        renderArgs.put("message", message);

        render(rooms.toArray(), teachers.toArray(), times.toArray(), weeks.toArray(), days.toArray(), classes.toArray(), message);
    }

    /**
     * Метод создания занятия.
     * @param s_name Название занятия.
     * @param s_lec Лекционное занятие или нет.
     * @param s_room Аудитория в которой будет занятие.
     * @param s_teacher Преподаватель ведущий предмет.
     * @param s_time Пара на которой ведется предмет.
     * @param s_week Неделя на которой читается предмет.
     * @param s_day День в который читается предмет.
     * @param s_class Группы для которых ведется предмет.
     */
    public static void createNew (String s_name, boolean s_lec, String s_room, String s_teacher, String s_time,
                                  String s_week, String s_day, String[] s_class) {

        if (Database.findByWeekDayTimeRoom(s_week, s_day, s_time, s_room).size() == 0) {
            if (Database.isValidByTeacher(s_week, s_day, s_teacher)) {
                if (Database.isValidByTeacher(s_week, s_day, s_time, s_teacher)) {
                    if (Database.isValidByClasses(s_week, s_day, s_class)) {
                        if (Database.isValidByClasses(s_week, s_day, s_time, s_class)) {
                            if (Database.isValidByRoom(s_lec, s_room)) {
                                Database.createSubject(s_name, s_lec, s_room, s_teacher, s_time, s_week, s_day, s_class);
                                create(null);
                            } else {
                                create("Размеры аудитории не соответствуют занятию!");
                            }
                        } else {
                            create("У группы уже есть пара в это время!");
                        }
                    } else {
                        create("У группы уже есть 4 пары в этот день!");
                    }
                } else {
                   create("У преподавателя уже есть пара в это время!");
                }
            } else {
                create("У преподавателя уже больше 4 пар!");
            }
        } else {
            create("В данной аудитории уже идет занятие!");
        }

    }

    /**
     * Функция отображения формы удаления предмета.
     */
    public static void delete() {

        ArrayList<Subject> subjects = Database.readSubjects();

        renderArgs.put("subjects", subjects.toArray());
        render(subjects.toArray());
    }

    /**
     * Метод удаления предмета.
     * @param id Идентификатор удаляемого предмета.
     */
    public static void remove (String id) {
        Database.removeSubject(id);

        delete();
    }

    /**
     * Функция отображения детальной страницы ученого предмета.
     */
    public static void detail(String id) {

        Subject subject = Database.getSubjectById(Integer.parseInt(id));

        renderArgs.put("subject", subject);
        render(subject);
    }

    /**
     * Функция отображения формы изменения учебного предмета.
     */
    public static void edit() {
        render();
    }

}
