package controllers;

import models.Database;
import models.owl.Teacher;
import play.mvc.Controller;

import java.util.ArrayList;

/**
 * Контроллер секции преподавателей.
 * @author M. Navrotskiy
 * @version 1.0
 */
public class TeachersSection extends Controller {

    /**
     * Функция отображения главной страницы секции.
     */
    public static void index() {

        ArrayList<Teacher> teachers = Database.readTeachers();

        renderArgs.put("teachers", teachers.toArray());
        render(teachers.toArray());
    }

    /**
     * Функция отображения формы создания преподавателя.
     */
    public static void create() {
        render();
    }

    /**
     * Функция создания преподавателя.
     * @param t_name Фамилия преподавателя.
     */
    public static void createNew (String t_name) {
        Database.addTeacher(t_name);

        create();
    }

    /**
     * Функция отображения формы удаления преподавателей.
     */
    public static void delete() {
        ArrayList<Teacher> teachers = Database.readTeachers();

        renderArgs.put("teachers", teachers.toArray());
        render(teachers.toArray());
    }

    /**
     * Функция удаления преподавателя.
     * @param id Идентификатор преподавателя.
     */
    public static void remove (String id) {
        Database.removeTeacher(id);

        delete();
    }

    /**
     * Функция отображения детальной страницы преподавателя.
     */
    public static void detail (String id) {

        Teacher t = Database.getTeacherById(Integer.parseInt(id));

        renderArgs.put("t",t);
        render(t);

        render();
    }

    /**
     * Функция отображения формы изменения преподавателя.
     */
    public static void edit() {
        render();
    }

}
