package models;

import com.hp.hpl.jena.ontology.*;
import com.hp.hpl.jena.query.*;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;
import models.owl.*;
import models.owl.Class;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.Iterator;

/**
 * Класс базы данных приложения.
 * @author M. Navrotskiy
 * @version 1.0
 */
public class Database {

    /** Имя файла с онтологией. */
    private static String filename = "schedule2.owl";
    /** Модель онтологии. */
    private static OntModel model = null;
    /** Префикс онтологии (пространство имен) */
    private static String prefix = "http://www.owl-ontologies.com/Ontology1368808386.owl#";
    /** Запрос на поиск предметов по группе. */
    private static String fSubjectClass = "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>\n" +
            "PREFIX owl: <http://www.w3.org/2002/07/owl#>\n" +
            "PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>\n" +
            "PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>\n" +
            "SELECT *\n" +
            "WHERE {\n" +
            "\t?subjects\n" +
            "\t<http://www.owl-ontologies.com/Ontology1368808386.owl#s_cs_value>\n" +
            "\t<@_@_@>\n" + // Данное место будет заменено на uri группы.
            "}";
    /** Запрос на посик предметов по неделе. */
    private static String fSubjectWeek = "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>\n" +
            "PREFIX owl: <http://www.w3.org/2002/07/owl#>\n" +
            "PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>\n" +
            "PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>\n" +
            "SELECT *\n" +
            "WHERE {\n" +
            "\t?subjects\n" +
            "\t<http://www.owl-ontologies.com/Ontology1368808386.owl#s_w_s_value>\n" +
            "\t<@_@_@>\n" + // Данное место будет заменено на uri недели.
            "}";
    /** Запрос на поиск предметов по дню. */
    private static String fSubjectDay = "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>\n" +
            "PREFIX owl: <http://www.w3.org/2002/07/owl#>\n" +
            "PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>\n" +
            "PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>\n" +
            "SELECT *\n" +
            "WHERE {\n" +
            "\t?subjects\n" +
            "\t<http://www.owl-ontologies.com/Ontology1368808386.owl#s_d_t_value>\n" +
            "\t<@_@_@>\n" + // Данное место будет заменено на uri недели.
            "}";
    /** Запрос на поиск предметов по аудитории. */
    private static String fSubjectRoom = "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>\n" +
            "PREFIX owl: <http://www.w3.org/2002/07/owl#>\n" +
            "PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>\n" +
            "PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>\n" +
            "SELECT *\n" +
            "WHERE {\n" +
            "\t?subjects\n" +
            "\t<http://www.owl-ontologies.com/Ontology1368808386.owl#s_r_value>\n" +
            "\t<@_@_@>\n" + // Данное место будет заменено на uri аудитории.
            "}";
    /** Запрос на поиск предметов по преподавателю. */
    private static String fSubjectTeacher = "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>\n" +
            "PREFIX owl: <http://www.w3.org/2002/07/owl#>\n" +
            "PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>\n" +
            "PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>\n" +
            "SELECT *\n" +
            "WHERE {\n" +
            "\t?subjects\n" +
            "\t<http://www.owl-ontologies.com/Ontology1368808386.owl#s_t_value>\n" +
            "\t<@_@_@>\n" + // Данное место будет заменено на uri преподавателя.
            "}";
    /** Запрос на поиск предметов по группе и неделе. */
    private static String fSubjectClassWeek = "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>\n" +
                    "PREFIX owl: <http://www.w3.org/2002/07/owl#>\n" +
                    "PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>\n" +
                    "PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>\n" +
                    "SELECT *\n" +
                    "WHERE {{\n" +
                    "\t?subjects\n" +
                    "\t<http://www.owl-ontologies.com/Ontology1368808386.owl#s_cs_value>\n" +
                    "\t<@_@_@>\n" + // Данное место будет заменено на uri группы.
                    "}.{\n" +
                    "\t?subjects\n" +
                    "\t<http://www.owl-ontologies.com/Ontology1368808386.owl#s_w_s_value>\n" +
                    "\t<$_$_$>\n" + // Данное место будет заменено на uri недели.
                    "}}";
    /** Запрос на поиск предметов по группе и дню. */
    private static String fSubjectClassDay = "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>\n" +
            "PREFIX owl: <http://www.w3.org/2002/07/owl#>\n" +
            "PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>\n" +
            "PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>\n" +
            "SELECT *\n" +
            "WHERE {{\n" +
            "\t?subjects\n" +
            "\t<http://www.owl-ontologies.com/Ontology1368808386.owl#s_cs_value>\n" +
            "\t<@_@_@>\n" + // Данное место будет заменено на uri группы.
            "}.{\n" +
            "\t?subjects\n" +
            "\t<http://www.owl-ontologies.com/Ontology1368808386.owl#s_d_t_value>\n" +
            "\t<$_$_$>\n" + // Данное место будет заменено на uri дня.
            "}}";
    /** Запрос на поиск предметов по неделе и дню. */
    private static String fSubjectWeekDay = "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>\n" +
            "PREFIX owl: <http://www.w3.org/2002/07/owl#>\n" +
            "PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>\n" +
            "PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>\n" +
            "SELECT *\n" +
            "WHERE {{\n" +
            "\t?subjects\n" +
            "\t<http://www.owl-ontologies.com/Ontology1368808386.owl#s_w_s_value>\n" +
            "\t<@_@_@>\n" + // Данное место будет заменено на uri недели.
            "}.{\n" +
            "\t?subjects\n" +
            "\t<http://www.owl-ontologies.com/Ontology1368808386.owl#s_d_t_value>\n" +
            "\t<$_$_$>\n" + // Данное место будет заменено на uri дня.
            "}}";
    /** Запрос на поиск предметов по группе, неделе и дню.*/
    private static String fSubjectClassWeekDay = "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>\n" +
            "PREFIX owl: <http://www.w3.org/2002/07/owl#>\n" +
            "PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>\n" +
            "PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>\n" +
            "SELECT *\n" +
            "WHERE {{\n" +
            "\t?subjects\n" +
            "\t<http://www.owl-ontologies.com/Ontology1368808386.owl#s_cs_value>\n" +
            "\t<@_@_@>\n" + // Данное место будет заменено на uri группы.
            "}.{\n" +
            "\t?subjects\n" +
            "\t<http://www.owl-ontologies.com/Ontology1368808386.owl#s_w_s_value>\n" +
            "\t<$_$_$>\n" + // Данное место будет заменено на uri недели.
            "}.{\n" +
            "\t?subjects\n" +
            "\t<http://www.owl-ontologies.com/Ontology1368808386.owl#s_d_t_value>\n" +
            "\t<#_#_#>\n" + // Данное место будет заменене но uri дня.
            "}}";
    /** Запрос на поиск предмета по неделе, дню, времени а аудитории. */
    private static String fSubjectWeekDayTimeRoom = "PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>\n" +
            "PREFIX owl: <http://www.w3.org/2002/07/owl#>\n" +
            "PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>\n" +
            "PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>\n" +
            "SELECT *\n" +
            "WHERE {{\n" +
            "\t?subjects\n" +
            "\t<http://www.owl-ontologies.com/Ontology1368808386.owl#s_w_s_value>\n" +
            "\t<@_@_@>\n" + // Данное место будет заменено на uri недели.
            "}.{\n" +
            "\t?subjects\n" +
            "\t<http://www.owl-ontologies.com/Ontology1368808386.owl#s_d_t_value>\n" +
            "\t<$_$_$>\n" + // Данное место будет заменено на uri дня.
            "}.{\n" +
            "\t?subjects\n" +
            "\t<http://www.owl-ontologies.com/Ontology1368808386.owl#s_t_s_value>\n" +
            "\t<#_#_#>\n" + // Данное место будет заменено на uri пары.
            "}.{\n" +
            "\t?subjects\n" +
            "\t<http://www.owl-ontologies.com/Ontology1368808386.owl#s_r_value>\n" +
            "\t<%_%_%>\n" + // Данное место будет заменено на uri аудитории.
            "}}";

    /**
     * Метод открытия онтологии.
     */
    public static void openModel() {
        model = ModelFactory.createOntologyModel(OntModelSpec.OWL_DL_MEM);
        model.read(filename, "RDF/XML-ABBREV");
    }

    /**
     * Метод проверки модели на null.
     */
    private static void checkModel() {
        if (model == null) {
            openModel();
        }
    }

    /**
     * Метод получения учебной группы из онтологии по ее uri.
     * @param uri URI учебной группы в онтологии.
     * @return Учебная группа или null.
     */
    public static Class getClassByUri (String uri) {

        ArrayList<Class> classes = readClasses();
        Iterator<Class> it = classes.iterator();

        while (it.hasNext()) {
            Class c = it.next();

            if (c.getOwlURI().equals(uri)) {
                return c;
            }
        }

        return null;
    }

    /**
     * Метод получения учебной аудитории из онтологии по ее uri.
     * @param uri URI учебной аудитории в онтологии.
     * @return Учебная аудитория или null.
     */
    public static Room getRoomByUri (String uri) {

        ArrayList<Room> rooms = readRooms();
        Iterator<Room> it = rooms.iterator();

        while (it.hasNext()) {
            Room r = it.next();

            if (r.getOwlURI().equals(uri)) {
                return r;
            }
        }

        return null;
    }

    /**
     * Метод получения преподавателя из онтологии по его uri.
     * @param uri URI преподавателя.
     * @return Преподаватель или null.
     */
    public static Teacher getTeacherByUri (String uri) {

        ArrayList<Teacher> teachers = readTeachers();
        Iterator<Teacher> it = teachers.iterator();

        while (it.hasNext()) {
            Teacher t = it.next();

            if (t.getOwlURI().equals(uri)) {
                return t;
            }
        }

        return null;
    }

    /**
     * Метод чтения групп из онтологии.
     * @return Список учебных групп.
     */
    public static ArrayList<Class> readClasses() {
        ArrayList<Class> classes = new ArrayList<Class>();

        checkModel();

        OntClass c = model.getOntClass(prefix + "Class");

        for (ExtendedIterator<?extends OntResource> it = c.listInstances(); it.hasNext();) {
            OntResource or = it.next();
            String name = cutStringFromOntology(or.getPropertyValue(model.getOntProperty(prefix + "name")).toString());
            int id = Integer.parseInt(cutStringFromOntology(or.getPropertyValue(model.getOntProperty(prefix + "id")).toString()));

            classes.add(new models.owl.Class(id, name, or.getURI()));

        }

        return classes;
    }

    /**
     * Метод чтения предметов из онтологии.
     * @return Список предметов.
     */
    public static ArrayList<Subject> readSubjects() {

        ArrayList<Subject> subjects = new ArrayList<Subject>();

        checkModel();

        OntClass s = model.getOntClass(prefix + "Subject");

        for (ExtendedIterator<?extends OntResource> it = s.listInstances(); it.hasNext();) {
            OntResource or = it.next();
            String name = cutStringFromOntology(or.getPropertyValue(model.getOntProperty(prefix + "name")).toString());
            int id = Integer.parseInt(cutStringFromOntology(or.getPropertyValue(model.getOntProperty(prefix + "id")).toString()));
            String room = or.getPropertyValue(model.getOntProperty(prefix + "s_r_value")).toString();
            String teacher = or.getPropertyValue(model.getOntProperty(prefix + "s_t_value")).toString();
            String day = or.getPropertyValue(model.getOntProperty(prefix + "s_d_t_value")).toString();
            String lection = or.getPropertyValue(model.getOntProperty(prefix + "s_i_l_s_value")).toString();
            String time = or.getPropertyValue(model.getOntProperty(prefix + "s_t_s_value")).toString();
            String week = or.getPropertyValue(model.getOntProperty(prefix + "s_w_s_value")).toString();
            Boolean flag = false;

            if (lection.equals("http://www.owl-ontologies.com/Ontology1368808386.owl#YES")) {
                flag = true;
            }

            Subject subject = new Subject(id, name, flag, getRoomByUri(room), getTeacherByUri(teacher), SubjectTime.parseURI(time),
                    or.getURI(), SubjectWeek.parseURI(week), DaySubject.parseURI(day));

            for (ExtendedIterator<?extends RDFNode> jt = or.listPropertyValues(model.getOntProperty(prefix + "s_cs_value")); jt.hasNext();) {

                RDFNode rt = jt.next();
                String c = rt.toString();

                subject.appendClass(getClassByUri(c));
            }

            subjects.add(subject);

        }

        return subjects;
    }

    /**
     * Метод чтения аудиторий из онтологии.
     * @return Список аудиторий из онтологии.
     */
    public static ArrayList<Room> readRooms() {
        ArrayList<Room> rooms = new ArrayList<Room>();

        checkModel();

        OntClass r = model.getOntClass(prefix + "Room");

        for (ExtendedIterator<?extends OntResource> it = r.listInstances(); it.hasNext();) {
            OntResource or = it.next();
            String name = cutStringFromOntology(or.getPropertyValue(model.getOntProperty(prefix + "name")).toString());
            int id = Integer.parseInt(cutStringFromOntology(or.getPropertyValue(model.getOntProperty(prefix + "id")).toString()));
            String ontFlag = cutStringFromOntology(or.getPropertyValue(model.getOntProperty(prefix + "r_i_l_s_value")).toString());
            boolean flag = false;

            if (ontFlag.charAt(ontFlag.length() - 1) == 'Y') {
                flag = true;
            }

            rooms.add(new Room(id, name, flag, or.getURI()));
        }

        return rooms;
    }

    /**
     * Метод чтения преподавателей из онтологии.
     * @return Список преподавателей.
     */
    public static ArrayList<Teacher> readTeachers() {
        ArrayList<Teacher> teachers = new ArrayList<Teacher>();

        checkModel();

        OntClass t = model.getOntClass(prefix + "Teacher");

        for (ExtendedIterator<?extends OntResource> it = t.listInstances(); it.hasNext();) {
            OntResource or = it.next();
            String name = cutStringFromOntology(or.getPropertyValue(model.getOntProperty(prefix + "name")).toString());
            int id = Integer.parseInt(cutStringFromOntology(or.getPropertyValue(model.getOntProperty(prefix + "id")).toString()));

            teachers.add(new Teacher(id, name, "", or.getURI()));
        }

        return teachers;
    }

    /**
     * Метод получения предмета из онтологии по его uri.
     * @param uri uri искомого предмета.
     * @return Найденный предмет или null;
     */
    public static Subject getSubjectFromOwlByURI(String uri) {

        Subject result = null;
        ArrayList<Subject> subjects = readSubjects();
        Iterator<Subject> it = subjects.iterator();

        while (it.hasNext()) {
            Subject temp = it.next();
            if (uri.equals(temp.getOwlURI())) {
                result = temp;
                break;
            }
        }

        return result;

    }

    /**
     * Метод чтения из онтологии неделей.
     * @return Список неделей.
     */
    public static ArrayList<String> readWeeks() {
        ArrayList<String> weeks = new ArrayList<String>();

        checkModel();

        OntClass c = model.getOntClass(prefix + "WeekSubject");

        for (ExtendedIterator<?extends OntResource> it = c.listInstances(); it.hasNext();) {
            OntResource or = it.next();
            String name = or.getURI();
            name = name.replaceFirst("^.*#", "");

            if (name.contains("1")) {
                weeks.add("Первая");
            } else if (name.contains("2")) {
                weeks.add("Вторая");
            }

        }

        return weeks;
    }

    /**
     * Метод чтения учебных дней из онтологии.
     * @return Список учебных дней.
     */
    public static ArrayList<String> readDays() {
        ArrayList<String> days = new ArrayList<String>();
        String[] ds = new String[6];

        checkModel();

        OntClass c = model.getOntClass(prefix + "DaySubject");

        for (ExtendedIterator<?extends OntResource> it = c.listInstances(); it.hasNext();) {
            OntResource or = it.next();
            String name = or.getURI();
            name = name.replaceFirst("^.*#", "");

            if (name.contains("1")) {
                ds[0] = "Понедельник";
            } else if (name.contains("2")) {
                ds[1] = "Вторник";
            } else if (name.contains("3")) {
                ds[2] = "Среда";
            } else if (name.contains("4")) {
                ds[3] = "Четверг";
            } else if (name.contains("5")) {
                ds[4] = "Пятница";
            } else if (name.contains("6")) {
                ds[5] = "Суббота";
            }

        }

        for (int i = 0; i < 6; i++) {
            if (!ds[i].equals("") && !ds[i].equals(null)) {
                days.add(ds[i]);
            }
        }

        return days;
    }

    /**
     * Функция обрезания лишней информации из строки.
     * @param ontoStr Онтологическая строка с лишними символами.
     * @return "Чистая" строка.
     */
    private static String cutStringFromOntology (String ontoStr) {

        String result = "";

        result = ontoStr.replaceFirst("\\^+.*", "");

        return result;

    }

    /**
     * Метод получения учебной группы по идентификатору.
     * @param id Идентификатор искомой группы.
     * @return Найденная группа или null.
     */
    public static Class getClassById (int id) {
        ArrayList<Class> classes = readClasses();
        Iterator<Class> it = classes.iterator();

        while (it.hasNext()) {
            Class t = it.next();

            if (t.getId() == id) {
                return t;
            }
        }

        return null;
    }

    /**
     * Метод получения учебной аудитории по идентификатору.
     * @param id Идентификатор искомой аудитории.
     * @return Найденная учебная аудитория или null.
     */
    public static Room getRoomById (int id) {
        ArrayList<Room> rooms = readRooms();
        Iterator<Room> it = rooms.iterator();

        while (it.hasNext()) {
            Room r = it.next();

            if (r.getId() == id) {
                return r;
            }
        }

        return null;
    }

    /**
     * Метод получения преподавателя по идентификатору.
     * @param id Идентификатор искомого преподавателя.
     * @return Найденный преподаватель или null.
     */
    public static Teacher getTeacherById (int id) {
        ArrayList<Teacher> teachers = readTeachers();
        Iterator<Teacher> it = teachers.iterator();

        while (it.hasNext()) {
            Teacher t = it.next();

            if (t.getId() == id) {
                return t;
            }
        }

        return null;
    }

    public static void removeSubject (String id) {
        checkModel();

        OntClass s = model.getOntClass(prefix + "Subject");
        ExtendedIterator<?extends OntResource> it = s.listInstances();

        while (it.hasNext()) {
            OntResource or = it.next();
            String rid = cutStringFromOntology(or.getPropertyValue(model.getOntProperty(prefix + "id")).toString());

            if (rid.equals(id)) {

                or.remove();
                break;
            }
        }

        try {
            model.write(new FileOutputStream(filename), "RDF/XML-ABBREV");
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
    }

    /**
     * Удаление учебной аудитории из онтологии.
     * @param id Идентификатор аудитории.
     */
    public static void removeRoom (String id) {
        checkModel();

        OntClass r = model.getOntClass(prefix + "Room");
        ExtendedIterator<?extends OntResource> it = r.listInstances();

        while (it.hasNext()) {
            OntResource or = it.next();
            String rid = cutStringFromOntology(or.getPropertyValue(model.getOntProperty(prefix + "id")).toString());

            if (rid.equals(id)) {
                Room rr = getRoomById(Integer.parseInt(id));
                OntResource resource = model.createOntResource(rr.getOwlURI());

                if (findByRoom(rr.getId()).size() != 0) {
                    return;
                }

                resource.remove();
                break;
            }
        }

        try {
            model.write(new FileOutputStream(filename), "RDF/XML-ABBREV");
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
    }


    /**
     * Метод удаления преподавателя из онтологии.
     * @param id Идентификатор преподавателя.
     */
    public static void removeTeacher (String id) {

        checkModel();

        OntClass t = model.getOntClass(prefix + "Teacher");
        ExtendedIterator<?extends OntResource> it = t.listInstances();

        while (it.hasNext()) {
            OntResource or = it.next();
            String tid = cutStringFromOntology(or.getPropertyValue(model.getOntProperty(prefix + "id")).toString());

            if (tid.equals(id)) {
                Teacher tt = getTeacherById(Integer.parseInt(id));
                OntResource resource = model.createOntResource(tt.getOwlURI());

                if (findByTeacher(tt.getId()).size() != 0) {
                    return;
                }

                resource.remove();
                break;
            }
        }

        try {
            model.write(new FileOutputStream(filename), "RDF/XML-ABBREV");
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }

    }

    /**
     * Метод удаления учебной группы.
     * @param id Идентификатор удаляемой учебной группы.
     */
    public static void removeClass (String id) {

        checkModel();

        OntClass c = model.getOntClass(prefix + "Class");
        ExtendedIterator<?extends OntResource> it = c.listInstances();

        while (it.hasNext()) {
            OntResource or = it.next();
            String cid = cutStringFromOntology(or.getPropertyValue(model.getOntProperty(prefix + "id")).toString());

            if (cid.equals(id)) {

                Class cc = getClassById(Integer.parseInt(id));
                OntResource resource = model.createOntResource(cc.getOwlURI());

                if (findByClass(cc.getId()).size() != 0) {
                    return;
                }

                resource.remove();
                break;
            }

        }

        try {
            model.write(new FileOutputStream(filename), "RDF/XML-ABBREV");
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }

    }

    /**
     * Функция добавления новой учебной аудитории в онтологию.
     * @param number Номер учебной аудитории.
     * @param flag Признак того, лекционная аудитория или нет.
     */
    public static void addRoom (String number, boolean flag) {
        ArrayList<Room> res = readRooms();
        int globalID = 0;

        for (int i = 0; i < res.size(); i++) {
            if (res.get(i).getId() > globalID) {
                globalID = res.get(i).getId();
            }
        }

        globalID++;
        String uri = prefix + "Room_" + Integer.toString(globalID);
        Room newR = new Room(globalID, number, flag, uri);

        checkModel();

        OntClass r = model.getOntClass(prefix + "Room");

        Individual ri = model.createIndividual(newR.getOwlURI(), r);
        ri.addProperty(model.getDatatypeProperty(prefix + "name"), newR.getNumber());
        ri.addProperty(model.getDatatypeProperty(prefix + "id"), Integer.toString(newR.getId()));
        String pUri = "http://www.owl-ontologies.com/Ontology1368808386.owl#N";
        if (newR.isForLections()) {
            pUri = "http://www.owl-ontologies.com/Ontology1368808386.owl#Y";
        }
        ri.addProperty(model.getObjectProperty(prefix + "r_i_l_s_value"), pUri);

        try {
            model.write(new FileOutputStream(filename), "RDF/XML-ABBREV");
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
    }

    /**
     * Метод добавления преподавателя в онтологию.
     * @param name Фамилия преподавателя.
     */
    public static void addTeacher (String name) {
        ArrayList<Teacher> tes = readTeachers();
        int globalID = 0;

        for (int i = 0; i < tes.size(); i++) {
            if (tes.get(i).getId() > globalID) {
                globalID = tes.get(i).getId();
            }
        }
        globalID++;

        String uri = prefix + "Teacher_" + Integer.toString(globalID);
        Teacher newT = new Teacher(globalID, name, "", uri);

        checkModel();

        OntClass t = model.getOntClass(prefix + "Teacher");

        Individual ti = model.createIndividual(newT.getOwlURI(), t);
        ti.addProperty(model.getDatatypeProperty(prefix + "name"), newT.getName());
        ti.addProperty(model.getDatatypeProperty(prefix + "id"), Integer.toString(newT.getId()));

        try {
            model.write(new FileOutputStream(filename), "RDF/XML-ABBREV");
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
    }

    /**
     * Метод добавления учебной группы в онтологию.
     * @param name Имя учебной группы.
     */
    public static void addClass (String name) {
        ArrayList<Class> ces = readClasses();
        int globalID = 0;

        for (int i = 0; i < ces.size(); i++) {
            if (ces.get(i).getId() > globalID) {
                globalID = ces.get(i).getId();
            }
        }
        globalID++;
        String uri = prefix + "Class_" + Integer.toString(globalID);
        Class newC = new Class(globalID, name, uri);

        checkModel();

        OntClass c = model.getOntClass(prefix + "Class");

        Individual ci = model.createIndividual(newC.getOwlURI(),c);
        ci.addProperty(model.getDatatypeProperty(prefix + "name"), newC.getName());
        ci.addProperty(model.getDatatypeProperty(prefix + "id"), Integer.toString(newC.getId()));

        try {
            model.write(new FileOutputStream(filename), "RDF/XML-ABBREV");
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
    }

    /**
     * Метод поиска предметов по аудитории.
     * @param id Идентификатор аудитории.
     * @return Список предметов.
     */
    public static ArrayList<Subject> findByRoom (int id) {
        ArrayList<Subject> result = new ArrayList<Subject>();
        Room r = getRoomById(id);

        checkModel();

        String queryString = fSubjectRoom.replaceAll("@_@_@", r.getOwlURI());

        Query query = QueryFactory.create(queryString);
        QueryExecution qexec = QueryExecutionFactory.create(query, model);
        ResultSet results = qexec.execSelect();
        //ResultSetFormatter.out(System.out, results, query);
        while (results.hasNext()) {
            QuerySolution soln = results.nextSolution();
            RDFNode x = soln.get("subjects");
            String name = x.toString();

            result.add(getSubjectFromOwlByURI(name));
        }

        qexec.close();

        return result;
    }

    /**
     * Метод поиска учебных предметов по учебной группе.
     * @param id Идентификатор учебной группы.
     * @return Список предметов.
     */
    public static ArrayList<Subject> findByClass (int id) {
        ArrayList<Subject> result = new ArrayList<Subject>();
        Class c = getClassById(id);

        checkModel();

        String queryString = fSubjectClass.replaceAll("@_@_@", c.getOwlURI());

        Query query = QueryFactory.create(queryString);
        QueryExecution qexec = QueryExecutionFactory.create(query, model);
        ResultSet results = qexec.execSelect();
        //ResultSetFormatter.out(System.out, results, query) ;
        for ( ; results.hasNext() ; ) {
            QuerySolution soln = results.nextSolution();
            RDFNode x = soln.get("subjects");
            String name = x.toString();

            result.add(getSubjectFromOwlByURI(name));
        }

        qexec.close();

        return result;
    }

    /**
     * Метод получения списка предметов по преподавателю.
     * @param id Идентификатор преподавателя.
     * @return Список предметов.
     */
    public static ArrayList<Subject> findByTeacher (int id) {
        ArrayList<Subject> result = new ArrayList<Subject>();
        Teacher t = getTeacherById(id);

        checkModel();

        String queryString = fSubjectTeacher.replaceAll("@_@_@", t.getOwlURI());
        Query query = QueryFactory.create(queryString);
        QueryExecution queryExecution = QueryExecutionFactory.create(query, model);
        ResultSet resultSet = queryExecution.execSelect();
//        ResultSetFormatter.out(System.out, resultSet, query);
        while (resultSet.hasNext()) {
            QuerySolution querySolution = resultSet.nextSolution();
            RDFNode rdfNode = querySolution.get("subjects");
            String name = rdfNode.toString();

            result.add(getSubjectFromOwlByURI(name));
        }

        queryExecution.close();

        return result;
    }

    /**
     * Метод получения предметов по неделе.
     * @param week Значение недели.
     * @return Список предметов.
     */
    public static ArrayList<Subject> findByWeek (String week) {
        ArrayList<Subject> result = new ArrayList<Subject>();

        checkModel();

        String queryString = fSubjectWeek.replaceAll("@_@_@", SubjectWeek.toURI(week));
        Query query = QueryFactory.create(queryString);
        QueryExecution queryExecution = QueryExecutionFactory.create(query, model);
        ResultSet resultSet = queryExecution.execSelect();
//        ResultSetFormatter.out(System.out, resultSet, query);
        while (resultSet.hasNext()) {
            QuerySolution querySolution = resultSet.nextSolution();
            RDFNode rdfNode = querySolution.get("subjects");
            String uri = rdfNode.toString();

            result.add(getSubjectFromOwlByURI(uri));
        }

        queryExecution.close();

        return result;
    }

    /**
     * Метод получения предметов по дню.
     * @param day Значение дня.
     * @return Список предметов.
     */
    public static ArrayList<Subject> findByDay (String day) {
        ArrayList<Subject> result = new ArrayList<Subject>();

        checkModel();

        String queryString = fSubjectDay.replaceAll("@_@_@", DaySubject.toURI(day));
        Query query = QueryFactory.create(queryString);
        QueryExecution queryExecution = QueryExecutionFactory.create(query, model);
        ResultSet resultSet = queryExecution.execSelect();
//        ResultSetFormatter.out(System.out, resultSet, query);
        while (resultSet.hasNext()) {
            QuerySolution querySolution = resultSet.nextSolution();
            RDFNode rdfNode = querySolution.get("subjects");
            String uri = rdfNode.toString();

            result.add(getSubjectFromOwlByURI(uri));
        }

        return result;
    }

    /**
     * Метод поиска учебных предметов по группе и неделе.
     * @param id Идентификатор учебной группы.
     * @param week Значение учебной недели.
     * @return Список предметов.
     */
    public static ArrayList<Subject> findByClassWeek (int id, String week) {
        ArrayList<Subject> result = new ArrayList<Subject>();

        checkModel();

        String queryString = fSubjectClassWeek.replaceAll("@_@_@", getClassById(id).getOwlURI());
        queryString = queryString.replaceAll("\\$_\\$_\\$", SubjectWeek.toURI(week));
        Query query = QueryFactory.create(queryString);
        QueryExecution queryExecution = QueryExecutionFactory.create(query, model);
        ResultSet resultSet = queryExecution.execSelect();
//        ResultSetFormatter.out(System.out, resultSet, query);
        while (resultSet.hasNext()) {
            QuerySolution querySolution = resultSet.nextSolution();
            RDFNode rdfNode = querySolution.get("subjects");
            String uri = rdfNode.toString();

            result.add(getSubjectFromOwlByURI(uri));
        }

        return result;
    }

    /**
     * Метод поиска предметов по группе и дню.
     * @param id Идентификатор учебной группы.
     * @param day Значение учебного дня.
     * @return Список учебных предметов.
     */
    public static ArrayList<Subject> findByClassDay (int id, String day) {
        ArrayList<Subject> result = new ArrayList<Subject>();

        checkModel();

        String queryString = fSubjectClassDay.replaceAll("@_@_@", getClassById(id).getOwlURI());
        queryString = queryString.replaceAll("\\$_\\$_\\$", DaySubject.toURI(day));
        Query query = QueryFactory.create(queryString);
        QueryExecution queryExecution = QueryExecutionFactory.create(query, model);
        ResultSet resultSet = queryExecution.execSelect();
//        ResultSetFormatter.out(System.out, resultSet, query);
        while (resultSet.hasNext()) {
            QuerySolution querySolution = resultSet.nextSolution();
            RDFNode rdfNode = querySolution.get("subjects");
            String uri = rdfNode.toString();

            result.add(getSubjectFromOwlByURI(uri));
        }

        return result;
    }

    /**
     * Метод поиска предметов по неделе и дню.
     * @param week Значение недели.
     * @param day Значение учебного дня.
     * @return Список учебных предметов.
     */
    public static ArrayList<Subject> findByWeekDay (String week, String day) {
        ArrayList<Subject> result = new ArrayList<Subject>();

        checkModel();

        String queryString = fSubjectWeekDay.replaceAll("@_@_@", SubjectWeek.toURI(week));
        queryString = queryString.replaceAll("\\$_\\$_\\$", DaySubject.toURI(day));
        Query query = QueryFactory.create(queryString);
        QueryExecution queryExecution = QueryExecutionFactory.create(query, model);
        ResultSet resultSet = queryExecution.execSelect();
//        ResultSetFormatter.out(System.out, resultSet, query);
        while (resultSet.hasNext()) {
            QuerySolution querySolution = resultSet.nextSolution();
            RDFNode rdfNode = querySolution.get("subjects");
            String uri = rdfNode.toString();

            result.add(getSubjectFromOwlByURI(uri));
        }

        return result;
    }

    /**
     * Метод поиска предметов по группе, неделе, дню.
     * @param id Идентификатор учебной группы.
     * @param week Значение недели.
     * @param day Значение учебного дня.
     * @return Список учебных предметов.
     */
    public static ArrayList<Subject> findByClassWeekDay (int id, String week, String day) {
        ArrayList<Subject> result = new ArrayList<Subject>();

        checkModel();

        String queryString = fSubjectClassWeekDay.replaceAll("@_@_@", getClassById(id).getOwlURI());
        queryString = queryString.replaceAll("\\$_\\$_\\$", SubjectWeek.toURI(week));
        queryString = queryString.replaceAll("#_#_#", DaySubject.toURI(day));
        Query query = QueryFactory.create(queryString);
        QueryExecution queryExecution = QueryExecutionFactory.create(query, model);
        ResultSet resultSet = queryExecution.execSelect();
//        ResultSetFormatter.out(System.out, resultSet, query);
        while (resultSet.hasNext()) {
            QuerySolution querySolution = resultSet.nextSolution();
            RDFNode rdfNode = querySolution.get("subjects");
            String uri = rdfNode.toString();

            result.add(getSubjectFromOwlByURI(uri));
        }

        return result;
    }

    /**
     * Метод чтения всех пар из онтологии.
     * @return Список пар.
     */
    public static ArrayList<String> readTime() {
        ArrayList<String> result = new ArrayList<String>();

        checkModel();

        OntClass t = model.getOntClass(prefix + "TimeSubject");
        ExtendedIterator<?extends OntResource> it = t.listInstances();
        while (it.hasNext()) {
            OntResource or = it.next();

            result.add(SubjectTime.toString(SubjectTime.parseURI(or.getURI())));
        }

        return result;
    }

    /**
     * Метод создания занятия.
     * @param s_name Название занятия.
     * @param s_lec Лекционное занятие или нет.
     * @param s_room Аудитория в которой будет занятие.
     * @param s_teacher Преподаватель ведущий предмет.
     * @param s_time Пара на которой ведется предмет.
     * @param s_week Неделя на которой читается предмет.
     * @param s_day День в который читается предмет.
     * @param s_class Группы для которых ведется предмет.
     */
    public static void createSubject (String s_name, boolean s_lec, String s_room, String s_teacher, String s_time,
                                      String s_week, String s_day, String[] s_class) {
        ArrayList<Subject> subjects = readSubjects();
        int globalID = 0;
        Iterator<Subject> i = subjects.iterator();

        while (i.hasNext()) {
            Subject temp = i.next();

            if (temp.getId() > globalID) {
                globalID = temp.getId();
            }
        }

        globalID++;
        String uri = prefix + "Subject_" + Integer.toString(globalID);
        Room r = getRoomById(Integer.parseInt(s_room));
        Teacher t = getTeacherById(Integer.parseInt(s_teacher));

        checkModel();

        OntClass s = model.getOntClass(prefix + "Subject");

        Individual si = model.createIndividual(uri, s);
        si.addProperty(model.getDatatypeProperty(prefix + "name"), s_name);
        si.addProperty(model.getDatatypeProperty(prefix + "id"), Integer.toString(globalID));
        String lec = s_lec ? "http://www.owl-ontologies.com/Ontology1368808386.owl#YES"
                : "http://www.owl-ontologies.com/Ontology1368808386.owl#NO";
        si.addProperty(model.getObjectProperty(prefix + "s_i_l_s_value"), lec);
        si.addProperty(model.getObjectProperty(prefix + "s_r_value"), r.getOwlURI());
        si.addProperty(model.getObjectProperty(prefix + "s_t_value"), t.getOwlURI());
        si.addProperty(model.getObjectProperty(prefix + "s_t_s_value"), SubjectTime.toURI(s_time));
        si.addProperty(model.getObjectProperty(prefix + "s_w_s_value"), SubjectWeek.toURI(s_week));
        si.addProperty(model.getObjectProperty(prefix + "s_d_t_value"), DaySubject.toURI(s_day));
        for (int j = 0; j < s_class.length; j++) {
            si.addProperty(model.getObjectProperty(prefix + "s_cs_value"), getClassById(Integer.parseInt(s_class[j])).getOwlURI());
        }

        try {
            model.write(new FileOutputStream(filename), "RDF/XML-ABBREV");
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }

    }

    /**
     * Получение предметов по неделе, дню, паре, аудитории.
     * @param week Неделя.
     * @param day День занятия.
     * @param time Время проведения занятия.
     * @param id Аудитория.
     * @return Список предметов.
     */
    public static ArrayList<Subject> findByWeekDayTimeRoom (String week, String day, String time, String id) {
        ArrayList<Subject> result = new ArrayList<Subject>();

        checkModel();

        String queryString = fSubjectWeekDayTimeRoom.replaceAll("@_@_@", SubjectWeek.toURI(week))
                .replaceAll("\\$_\\$_\\$", DaySubject.toURI(day)).replaceAll("#_#_#", SubjectTime.toURI(time))
                .replaceAll("%_%_%", getRoomById(Integer.parseInt(id)).getOwlURI());
        Query query = QueryFactory.create(queryString);
        QueryExecution qexec = QueryExecutionFactory.create(query, model);
        ResultSet results = qexec.execSelect();
//        ResultSetFormatter.out(System.out, results, query);
        while (results.hasNext()) {
            QuerySolution soln = results.nextSolution();
            RDFNode x = soln.get("subjects");
            String name = x.toString();

            result.add(getSubjectFromOwlByURI(name));
        }

        qexec.close();

        return result;
    }

    /**
     * Метод проверки что у преподавателя не больше 4 пар в день.
     * @param week Неделя.
     * @param day День.
     * @param id Идентификатор преподавателя.
     * @return true если все ок.
     */
    public static boolean isValidByTeacher (String week, String day, String  id) {

        ArrayList<Subject> subjects = readSubjects();
        Iterator<Subject> it = subjects.iterator();
        int count = 0;

        while (it.hasNext()) {
            Subject subject = it.next();

            if (subject.getWeekString().equals(week) &&
                    subject.getDayString().equals(day) &&
                    subject.getTeacher().getId() == Integer.parseInt(id)) {
                count++;
            }

        }

        if (count <= 4)
            return true;
        else
            return false;
    }

    /**
     * Метод проверки, что у преподавателя не более 1 пары в одно время.
     * @param week Неделя
     * @param day День.
     * @param time Пара.
     * @param id Идентификатор преподавателя.
     * @return true если проблем нет.
     */
    public static boolean isValidByTeacher (String week, String day, String time, String id) {

        ArrayList<Subject> subjects = readSubjects();
        Iterator<Subject> it = subjects.iterator();
        int count = 0;

        while (it.hasNext()) {
            Subject subject = it.next();

            if (subject.getWeekString().equals(week) &&
                    subject.getDayString().equals(day) &&
                    subject.getTeacher().getId() == Integer.parseInt(id) &&
                    subject.getTimeString().equals(time)) {
                    count++;
            }

        }

        if (count < 1)
            return true;
        else
            return false;

    }

    /**
     * Метод проверки что у группы не больше 4 пар в день.
     * @param week Неделя.
     * @param day День.
     * @param id Идентификатор группы.
     * @return true если все ок.
     */
    public static boolean isValidByClass (String week, String day, String id) {

        ArrayList<Subject> subjects = readSubjects();
        Iterator<Subject> it = subjects.iterator();
        int count = 0;

        while (it.hasNext()) {
            Subject subject = it.next();

            if (subject.getWeekString().equals(week) &&
                    subject.getDayString().equals(day) &&
                    subject.getClassById(Integer.parseInt(id)) != null) {
                count++;
            }

        }

        if (count <= 4)
            return true;
        else
            return false;
    }

    /**
     * Метод проверки, что у группы не более 1 пары в одно время.
     * @param week Неделя
     * @param day День.
     * @param time Пара.
     * @param id Идентификатор группы.
     * @return true если проблем нет.
     */
    public static boolean isValidByClass (String week, String day, String time, String id) {

        ArrayList<Subject> subjects = readSubjects();
        Iterator<Subject> it = subjects.iterator();
        int count = 0;

        while (it.hasNext()) {
            Subject subject = it.next();

            if (subject.getWeekString().equals(week) &&
                    subject.getDayString().equals(day) &&
                    subject.getClassById(Integer.parseInt(id)) != null &&
                    subject.getTimeString().equals(time)) {
                count++;
            }

        }

        if (count <= 1)
            return true;
        else
            return false;
    }

    /**
     * Метод проверки что у групп не больше 4 пар в день.
     * @param week Неделя.
     * @param day День.
     * @param id Идентификаторы групп.
     * @return true если все ок.
     */
    public static boolean isValidByClasses (String week, String day, String[] id) {

        boolean result = true;

        for (int i = 0; i < id.length; i++) {
            if (!isValidByClass(week, day, id[i]))
                result = false;
        }

        return result;
    }

    /**
     * Метод проверки, что у групп не более 1 пары в одно время.
     * @param week Неделя
     * @param day День.
     * @param time Пара.
     * @param id Идентификаторы групп.
     * @return true если проблем нет.
     */
    public static boolean isValidByClasses (String week, String day,String time, String[] id) {

        boolean result = true;

        for (int i = 0; i < id.length; i++) {
            if (!isValidByClass(week, day, time, id[i]))
                result = false;
        }

        return result;
    }

    /**
     * Метод проверки соответствия размеров аудитории.
     * @param lec Лекция или нет.
     * @param room Аудитория для занятия.
     * @return true, если проблем нет.
     */
    public static boolean isValidByRoom (boolean lec, String room) {

        if (lec && !getRoomById(Integer.parseInt(room)).isForLections()) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * Метод получения предмета по идентификатору.
     * @param id Идентификатор.
     * @return Предмет.
     */
    public static Subject getSubjectById (int id) {

        Iterator<Subject> it = readSubjects().iterator();

        while (it.hasNext()) {
            Subject t = it.next();

            if (t.getId() == id) {
                return t;
            }
        }

        return null;
    }

    /**
     * Метод изменения аудитории.
     * @param id Идентификатор аудитории.
     * @param number Номер аудитории.
     * @param lec Лекционная аудитория или нет.
     */
    public static void editRoom (int id, String number, boolean lec) {

        checkModel();

        OntClass s = model.getOntClass(prefix + "Room");
        Individual individual = model.getIndividual(getRoomById(id).getOwlURI());
        individual.addProperty(model.getDatatypeProperty(prefix + "name"), number);
        String pUri = "http://www.owl-ontologies.com/Ontology1368808386.owl#N";
        if (lec) {
            pUri = "http://www.owl-ontologies.com/Ontology1368808386.owl#Y";
        }
        individual.addProperty(model.getObjectProperty(prefix + "r_i_l_s_value"), pUri);

        try {
            model.write(new FileOutputStream(filename), "RDF/XML-ABBREV");
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
    }

    /**
     * Метод изменения группы.
     * @param id Идентификатор группы.
     * @param name Название группы.
     */
    public static void editClass (int id, String name) {
        checkModel();

        OntClass c = model.getOntClass(prefix + "Class");
        Individual individual = model.getIndividual(getClassById(id).getOwlURI());
        individual.addProperty(model.getDatatypeProperty(prefix + "name"), name);

        try {
            model.write(new FileOutputStream(filename), "RDF/XML-ABBREV");
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
    }

}

