package models.owl;

/**
 * Класс - день недели, в который читается предмет.
 * @author M. Navrotskiy
 * @version 1.0
 */
public class DaySubject {

    /* Поля класса. */
    /** Понедельник. */
    public static int MONDAY    = 0x0701;
    /** Вторник. */
    public static int TUESDAY   = 0x0702;
    /** Среда. */
    public static int WEDNESDAY = 0x0703;
    /** Четверг. */
    public static int THURSDAY  = 0x0704;
    /** Пятница. */
    public static int FRIDAY    = 0x0705;
    /** Суббота. */
    public static int SATURDAY  = 0x0706;

    /**
     * Метод преобразования uri из онтологии в значение.
     * @param uri URI из онтологии.
     * @return Значение дня.
     */
    public static int parseURI (String uri) {

        if (uri.equals("http://www.owl-ontologies.com/Ontology1368808386.owl#d_1")) {
            return MONDAY;
        } else if (uri.equals("http://www.owl-ontologies.com/Ontology1368808386.owl#d_2")) {
            return TUESDAY;
        } else if (uri.equals("http://www.owl-ontologies.com/Ontology1368808386.owl#d_3")) {
            return WEDNESDAY;
        } else if (uri.equals("http://www.owl-ontologies.com/Ontology1368808386.owl#d_4")) {
            return THURSDAY;
        } else if (uri.equals("http://www.owl-ontologies.com/Ontology1368808386.owl#d_5")) {
            return FRIDAY;
        } else if (uri.equals("http://www.owl-ontologies.com/Ontology1368808386.owl#d_6")) {
            return SATURDAY;
        } else {
            return -1;
        }
    }

    /**
     * Метод преобразования численного представления в uri.
     * @param value Численное значение дня.
     * @return URI в онтологии.
     */
    public static String toURI (final int value) {
        if (value == MONDAY) {
            return "http://www.owl-ontologies.com/Ontology1368808386.owl#d_1";
        } else if (value == TUESDAY) {
            return "http://www.owl-ontologies.com/Ontology1368808386.owl#d_2";
        } else if (value == WEDNESDAY) {
            return "http://www.owl-ontologies.com/Ontology1368808386.owl#d_3";
        } else if (value == THURSDAY) {
            return "http://www.owl-ontologies.com/Ontology1368808386.owl#d_4";
        } else if (value == FRIDAY) {
            return "http://www.owl-ontologies.com/Ontology1368808386.owl#d_5";
        } else if (value == SATURDAY) {
            return "http://www.owl-ontologies.com/Ontology1368808386.owl#d_6";
        } else {
            return "";
        }
    }

    /**
     * Метод преобразования строкового представления в uri.
     * @param value Строковое значение дня.
     * @return URI в онтологии.
     */
    public static String toURI (final String value) {
        if (value.equals("Понедельник")) {
            return "http://www.owl-ontologies.com/Ontology1368808386.owl#d_1";
        } else if (value.equals("Вторник")) {
            return "http://www.owl-ontologies.com/Ontology1368808386.owl#d_2";
        } else if (value.equals("Среда")) {
            return "http://www.owl-ontologies.com/Ontology1368808386.owl#d_3";
        } else if (value.equals("Четверг")) {
            return "http://www.owl-ontologies.com/Ontology1368808386.owl#d_4";
        } else if (value.equals("Пятница")) {
            return "http://www.owl-ontologies.com/Ontology1368808386.owl#d_5";
        } else if (value.equals("Суббота")) {
            return "http://www.owl-ontologies.com/Ontology1368808386.owl#d_6";
        } else {
            return "";
        }
    }

    /**
     * Метод преобразования дня в строку.
     * @param value Значение дня.
     * @return Значение дня в виде строки.
     */
    public static String toString (int value) {

        if (value == MONDAY) {
            return "Понедельник";
        } else if (value == TUESDAY) {
            return "Вторник";
        } else if (value == WEDNESDAY) {
            return "Среда";
        } else if (value == THURSDAY) {
            return "Четверг";
        } else if (value == FRIDAY) {
            return "Пятница";
        } else if (value == SATURDAY) {
            return "Суббота";
        } else {
            return "Неизвестно";
        }
    }

}
