package models.owl;

/**
 * Класс учебной аудитории.
 * @author M. Navrotskiy
 * @version 1.0
 */
public class Room {

    /**
     * Конструктор.
     * @param id Идентификатор.
     * @param number Номер.
     * @param flag Признак того, что аудитория лекционнная.
     * @param owlURI uri экземпляра класса в онтологии.
     */
    public Room (int id, String number, boolean flag, String owlURI){
        this.id = id;
        this.isForLections = flag;
        this.number = number;
        this.owlURI = owlURI;
    }

    /**
     * Метод изменения номера аудитории.
     * @param number Номер аудории.
     */
    public void setNumber(String  number) {
        this.number = number;
    }

    /**
     * Метод изменения идентификатора аудитории.
     * @param id Идентификатор аудитории.
     */
    public void setId (int id) {
        this.id = id;
    }

    /**
     * Метод изменения признака лекционная ли это аудитория.
     * @param flag Признак лекционная ли это аудитория.
     */
    public void setForLections (boolean flag) {
        this.isForLections = flag;
    }

    /**
     * Метод изменения uri экземпляра класса в онтологии.
     * @param owlURI uri экземпляра класса в онтологии.
     */
    public void setOwlURI (String owlURI) {
        this.owlURI = owlURI;
    }

    /**
     * Метод получения признака лекционная ли это аудитория.
     * @return true, если аудитория лекционная, иначе false.
     */
    public boolean isForLections() {
        return this.isForLections;
    }

    /**
     * Метод полчения признака лекционная ли это аудитория.
     * @return Строковое представление значения.
     */
    public String forLections() {
        return this.isForLections ? "Да" : "Нет";
    }

    /**
     * Метод получения номера аудитории.
     * @return Номер аудитории.
     */
    public String getNumber() {
        return this.number;
    }

    /**
     * Метод получения идентификатора аудитории.
     * @return Идентификатор аудитории.
     */
    public int getId() {
        return this.id;
    }

    /**
     * Метод получения uri экземпляра класса в онтологии.
     * @return uri экземпляра класса в онтологии.
     */
    public String getOwlURI() {
        return this.owlURI;
    }

    /* Поля класса. */
    /** Идентификатор аудитории. */
    private int id;
    /** Номер аудитории. */
    private String number;
    /** Флаг того, лекционная это аудитория или нет. */
    private boolean isForLections;
    /** uri экземпляра класса в онтологии. */
    private String owlURI;
}
