package models.owl;

/**
 * Класс определяющий номер пары на которой идет предмет.
 * @author M. Navrotskiy
 * @version 1.0.
 */
public class SubjectTime {

    /** Пара 1 - 2 */
    public static int ST_12 = 0x0102;
    /** Пара 3 - 4 */
    public static int ST_34 = 0x0304;
    /** Пара 5 - 6 */
    public static int ST_56 = 0x0506;
    /** Пара 7 - 8 */
    public static int ST_78 = 0x0708;
    /** Пара 9 - 10 */
    public static int ST_910 = 0x0910;
    /** Пара 11 - 12 */
    public static int ST_1112 = 0x1112;

    /**
     * Метод преобразования uri из онтологии в значение.
     * @param uri URI из онтологии.
     * @return Значение пар.
     */
    public static int parseURI (String uri) {

        if (uri.equals("http://www.owl-ontologies.com/Ontology1368808386.owl#t_1-2")) {
            return ST_12;
        } else if (uri.equals("http://www.owl-ontologies.com/Ontology1368808386.owl#t_11-12")) {
            return ST_1112;
        } else if (uri.equals("http://www.owl-ontologies.com/Ontology1368808386.owl#t_3-4")) {
            return ST_34;
        } else if (uri.equals("http://www.owl-ontologies.com/Ontology1368808386.owl#t_5-6")) {
            return ST_56;
        } else if (uri.equals("http://www.owl-ontologies.com/Ontology1368808386.owl#t_7-8")) {
            return ST_78;
        } else if (uri.equals("http://www.owl-ontologies.com/Ontology1368808386.owl#t_9-10")) {
            return ST_910;
        } else {
            return -1;
        }
    }

    /**
     * Метод преобразования строкового представления в uri в онтологии.
     * @param value Строковое значение пары.
     * @return URI в онтологии.
     */
    public static String toURI (final String value) {
        if (value.equals("1-2")) {
            return "http://www.owl-ontologies.com/Ontology1368808386.owl#t_1-2";
        } else if (value.equals("3-4")) {
            return "http://www.owl-ontologies.com/Ontology1368808386.owl#t_3-4";
        } else if (value.equals("5-6")) {
            return "http://www.owl-ontologies.com/Ontology1368808386.owl#t_5-6";
        } else if (value.equals("7-8")) {
            return "http://www.owl-ontologies.com/Ontology1368808386.owl#t_7-8";
        } else if (value.equals("9-10")) {
            return "http://www.owl-ontologies.com/Ontology1368808386.owl#t_9-10";
        } else if (value.equals("11-12")) {
            return "http://www.owl-ontologies.com/Ontology1368808386.owl#t_11-12";
        } else {
            return "";
        }
    }

    /**
     * Метод преобразования номера пар в строку.
     * @param tm Номер пар.
     * @return Строка номера пар.
     */
    public static String toString(int tm) {

        if (tm == ST_12) {
            return "1-2";
        } else if (tm == ST_34) {
            return "3-4";
        } else if (tm == ST_56) {
            return "5-6";
        } else if (tm == ST_78) {
            return "7-8";
        } else if (tm == ST_910) {
            return "9-10";
        } else if (tm == ST_1112) {
            return "11-12";
        }

        return "";
    }

}
