package models.owl;

/**
 * Класс недели проведения занятия.
 * @author M. Navrotskiy
 * @version 1.0
 */
public class SubjectWeek {

    /* Поля класса. */
    /** Первая неделя. */
    public static int FIRST = 0x01;
    /** Вторая неделя. */
    public static int SECOND = 0x02;

    /**
     * Метод преобразования uri из онтологии в значение.
     * @param uri URI из онтологии.
     * @return Значение.
     */
    public static int parseURI (String uri) {

        if (uri.equals("http://www.owl-ontologies.com/Ontology1368808386.owl#w_1")) {
            return FIRST;
        } else if (uri.equals("http://www.owl-ontologies.com/Ontology1368808386.owl#w_2")) {
            return SECOND;
        } else {
            return -1;
        }
    }

    /**
     * Метод преобразования численного представления в URI.
     * @param value Численное значение недели.
     * @return URI в онтологии.
     */
    public static String toURI (final int value) {
        if (value == FIRST) {
            return "http://www.owl-ontologies.com/Ontology1368808386.owl#w_1";
        } else if (value == SECOND) {
            return "http://www.owl-ontologies.com/Ontology1368808386.owl#w_2";
        } else {
            return "";
        }
    }

    /**
     * Метод преобразования строкового представления в URI.
     * @param value Строковое значение недели.
     * @return URI в онтологии.
     */
    public static String toURI (final String value) {
        if (value.equals("Первая")) {
            return "http://www.owl-ontologies.com/Ontology1368808386.owl#w_1";
        } else if (value.equals("Вторая")) {
            return "http://www.owl-ontologies.com/Ontology1368808386.owl#w_2";
        } else  {
            return "";
        }
    }

    /**
     * Метод преобразования недели в строку.
     * @param week Неделя.
     * @return Строковое значение недели.
     */
    public static String toString(int week) {

        if (week == FIRST) {
            return "Первая";
        } else if (week == SECOND) {
            return "Вторая";
        }

        return "Неизвестно";
    }

}
